
import { useCallback, useEffect, useState } from "react";

import { TaskAssignment } from "adapters/task";

import Popup from 'components/ui/Popup';
import Button from 'components/ui/Button';
import { PossibleMember } from "components/forms/TaskForm";
import TimeInput from "components/ui/TimeInput";

import './assignees-form.scss';

interface Props {
    assignees: TaskAssignment[];
    members: PossibleMember[]
    onNew: (req: TaskAssignment) => any,
    onDelete: (req: string) => any
}

export default function AssigneesForm({ assignees, members, onNew, onDelete }: Props) {
    const [possibleMembers, setPossibleMembers] = useState<PossibleMember[]>([]);
    const [addNew, setAddNew] = useState(false);
    const [selectedMember, setSelectedMember] = useState('');
    const [selectedTime, setSelectedTime] = useState(Number.NaN);

    useEffect(() => {
        setPossibleMembers(members.filter(member => !assignees.find(r => r.user === member.id)));
    }, [members, assignees])

    const addAssignee = useCallback((e) => {
        e.preventDefault();
        if (!Number.isNaN(selectedTime) && selectedMember) {
            onNew({
                user: selectedMember,
                time: selectedTime * 60,
                finished: false,
            });
            setAddNew(false);
            setSelectedMember('');
            setSelectedTime(Number.NaN);

        }
    }, [selectedMember, selectedTime, onNew])

    const removeAssignee = useCallback((member: string) => {
        onDelete(member);
    }, [onDelete])

    return (
        <>
            <div className="assignees-field">
                <h2>Assignees</h2>
                {
                    assignees.map((assignee) => (
                        <div className="assignee" key={assignee.user}>
                            <div className="person">
                                {members.find(member => member.id === assignee.user)?.label}
                            </div>
                            <div className="time">{assignee.time} min</div>
                            <div className="delete" onClick={() => removeAssignee(assignee.user)}>
                                <span className="material-icons">
                                    clear
                                </span>
                            </div>
                        </div>
                    ))
                }
                {
                    possibleMembers.length > 0 && (
                        <div className="add-btn assignee" onClick={() => setAddNew(true)}>
                            +
                        </div>
                    )
                }
            </div>
            {
                addNew && (
                    <Popup onClose={() => setAddNew(false)}>
                        <select onChange={(e) => setSelectedMember(e.target.value)}>
                            <option value="">Please select a user</option>
                            {
                                possibleMembers.map((member) => (
                                    <option value={member.id} key={member.id}>{member.label}</option>
                                ))
                            }
                        </select>
                        <TimeInput onChange={value => setSelectedTime(value)} />
                        <Button type="submit" onClick={addAssignee} className="Expanded">
                            Add the assignee
                        </Button>
                    </Popup>
                )
            }
        </>
    )
}
